// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https ://mozilla.org/MPL/2.0/.

#ifndef ANGLE_H
#define ANGLE_H

namespace Passer {
namespace LinearAlgebra {

static float pi = 3.1415927410125732421875F;

static float Rad2Deg = 360.0f / (pi * 2);
static float Deg2Rad = (pi * 2) / 360.0f;

template <typename T>
class AngleOf {
 public:
  AngleOf() {};
  AngleOf(int f);
  AngleOf(float f);

  static AngleOf<T> Degrees(float f);
  static AngleOf<T> Radians(float f);

  float ToFloat() const;
  float InDegrees() const;
  float InRadians() const;

  inline T GetBinary() const { return this->value; }

  // static AngleOf<T> pi;

  bool operator==(AngleOf<T> a);
  bool operator>(AngleOf<T> a);
  bool operator>=(AngleOf<T> a);
  bool operator<(AngleOf<T> a);
  bool operator<=(AngleOf<T> a);

  AngleOf<T> operator-() const;
  AngleOf<T> operator-(const AngleOf<T>& a) const;
  AngleOf<T> operator+(const AngleOf<T>& a) const;
  AngleOf<T> operator+=(const AngleOf<T>& a);

  friend AngleOf<T> operator*(const AngleOf<T>& a, float f) {
    return AngleOf((float)a.ToFloat() * f);
  }
  friend AngleOf<T> operator*(float f, const AngleOf<T>& a) {
    return AngleOf((float)f * a.ToFloat());
  }

  static AngleOf<T> Normalize(AngleOf<T> a);
  static AngleOf<T> Clamp(AngleOf<T> a, AngleOf<T> min, AngleOf<T> max);
  static AngleOf<T> Difference(AngleOf<T> a, AngleOf<T> b) {
    AngleOf<T> r = Normalize(b.ToFloat() - a.ToFloat());
    return r;
  };
  static AngleOf<T> MoveTowards(AngleOf<T> fromAngle,
                                AngleOf<T> toAngle,
                                AngleOf<T> maxAngle);

  static float Cos(AngleOf<T>);
  static float Sin(AngleOf<T>);
  static float Tan(AngleOf<T>);

  static AngleOf<T> CosineRuleSide(float a, float b, AngleOf<T> gamma);
  static AngleOf<T> CosineRuleAngle(float a, float b, float c);

  static AngleOf<T> SineRuleAngle(float a, AngleOf<T> beta, float c);

 private:
  T value;
};

using Angle = AngleOf<float>;
using AngleSingle = AngleOf<float>;
using Angle16 = AngleOf<signed short>;

}  // namespace LinearAlgebra
}  // namespace Passer
using namespace Passer::LinearAlgebra;

#endif